package spring.aop.simulation.proxy;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.List;

import spring.aop.simulation.proxy.advice.Interceptor;
import spring.aop.simulation.proxy.advice.ReflectiveMethodInvocation;

public class JdkDynamicAopProxy implements AopProxy, InvocationHandler {
    private final Object target;
    private final List<Interceptor> advised;
    
    public JdkDynamicAopProxy(Object target, List<Interceptor> advised) {
        this.target = target;
        this.advised = advised;
    }
    
    @Override
    public Object getProxy() {
        return Proxy.newProxyInstance(target.getClass().getClassLoader(), target.getClass().getInterfaces(), this);
    }
    
    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        if (advised == null) {
            return method.invoke(target, args);
        } else {
            ReflectiveMethodInvocation invocation = new ReflectiveMethodInvocation(target, method, args, advised);
            
            return invocation.proceed();
        }
    }
}