package spring.aop.simulation.proxy.advice;

import java.lang.reflect.Method;
import java.util.List;

public class ReflectiveMethodInvocation implements MethodInvocation {
	private final Object target;
	private final Method method;
	private final Object[] args;
	private final List<Interceptor> advised;
	private int currentInterceptorIndex = -1;
	
    public ReflectiveMethodInvocation(Object target, Method method, Object[] args, List<Interceptor> advised) {
		this.target = target;
		this.advised = advised;
		this.method = method;
		this.args = args;
	}

    @Override
	public Object proceed() throws Throwable {
		if (currentInterceptorIndex < advised.size() - 1) {
			Interceptor interceptor = advised.get(++currentInterceptorIndex);
			
			return interceptor.invoke(this);
		} else {
			return method.invoke(target, args);
		}
    }
}